@@ -470,6 +470,34 @@ void Fl_File_Chooser_set_hidden_label(const char *msg) {
     LOCK(Fl_File_Chooser::hidden_label = msg)
 }
 
+void Fl_File_Chooser_set_position(Fl_File_Chooser *self, int x, int y) {
+    LOCK(self->position(x, y));
+}
+
+void Fl_File_Chooser_set_size(Fl_File_Chooser *self, int w, int h) {
+    LOCK(self->size(w, h));
+}
+
+int Fl_File_Chooser_x(const Fl_File_Chooser *self) {
+    LOCK(auto ret = self->x());
+    return ret;
+}
+
+int Fl_File_Chooser_y(const Fl_File_Chooser *self) {
+    LOCK(auto ret = self->y());
+    return ret;
+}
+
+int Fl_File_Chooser_w(const Fl_File_Chooser *self) {
+    LOCK(auto ret = self->w());
+    return ret;
+}
+
+int Fl_File_Chooser_h(const Fl_File_Chooser *self) {
+    LOCK(auto ret = self->h());
+    return ret;
+}
+
 char *Fl_dir_chooser(const char *message, const char *fname, int relative) {
     LOCK(auto ret = fl_dir_chooser(message, fname, relative));
     return ret;
@@ -494,4 +522,26 @@ void Fl_message_title_default(const char *title) {
     LOCK(fl_message_title_default(title));
 }
 
+void *Fl_message_icon(void) {
+    LOCK(auto ret = fl_message_icon());
+    return ret;
+}
+
+void Fl_message_set_hotspot(int enable) {
+    LOCK(fl_message_hotspot(enable));
+}
+
+int Fl_message_hotspot(void) {
+    LOCK(auto ret = fl_message_hotspot());
+    return ret;
+}
+
+void Fl_message_set_font(int f, int s) {
+    LOCK(fl_message_font((Fl_Font)f, (Fl_Fontsize)s));
+}
+
+void Fl_message_icon_label(const char *label) {
+    LOCK(fl_message_icon_label(label));
+}
+
 #undef LOCK
