@@ -74,12 +74,12 @@
         LOCK(if (boolean) self->show_cursor(); else self->hide_cursor());                          \
     }                                                                                              \
     void widget##_set_highlight_data(widget *self, void *sbuff, unsigned int *color, int *font,    \
-                                     int *fontsz, int sz) {                                        \
+                                     int *fontsz, unsigned *attr, unsigned int *bgcolor, int sz) { \
         Fl_Text_Display::Style_Table_Entry *stable = new Fl_Text_Display::Style_Table_Entry[sz];   \
         if (!stable)                                                                               \
             return;                                                                                \
         for (int i = 0; i < sz; ++i) {                                                             \
-            stable[i] = {color[i], font[i], fontsz[i], 0};                                         \
+            stable[i] = {color[i], font[i], fontsz[i], attr[i], bgcolor[i]};                       \
         }                                                                                          \
         LOCK(self->highlight_data((Fl_Text_Buffer *)sbuff, stable, sz, 'A', 0, 0));                \
     }                                                                                              \
@@ -205,6 +205,27 @@
     int widget##_wrapped_row(const widget *self, int row) {                                        \
         LOCK(auto ret = self->wrapped_row(row));                                                   \
         return ret;                                                                                \
+    }                                                                                              \
+    void widget##_set_grammar_underline_color(widget *self, unsigned int color) {                  \
+        LOCK(self->grammar_underline_color((Fl_Color)color));                                      \
+    }                                                                                              \
+    unsigned int widget##_grammar_underline_color(const widget *self) {                            \
+        LOCK(auto ret = self->grammar_underline_color());                                          \
+        return ret;                                                                                \
+    }                                                                                              \
+    void widget##_set_spelling_underline_color(widget *self, unsigned int color) {                 \
+        LOCK(self->spelling_underline_color((Fl_Color)color));                                     \
+    }                                                                                              \
+    unsigned int widget##_spelling_underline_color(const widget *self) {                           \
+        LOCK(auto ret = self->spelling_underline_color());                                         \
+        return ret;                                                                                \
+    }                                                                                              \
+    void widget##_set_secondary_selection_color(widget *self, unsigned int color) {                \
+        LOCK(self->secondary_selection_color((Fl_Color)color));                                    \
+    }                                                                                              \
+    unsigned int widget##_secondary_selection_color(const widget *self) {                          \
+        LOCK(auto ret = self->secondary_selection_color());                                        \
+        return ret;                                                                                \
     }
 
 Fl_Text_Buffer *Fl_Text_Buffer_new(void) {
@@ -316,6 +337,37 @@ void Fl_Text_Buffer_replace_selection(Fl_Text_Buffer *self, const char *text) {
     LOCK(self->replace_selection(text));
 }
 
+void Fl_Text_Buffer_secondary_select(Fl_Text_Buffer *self, int start, int end) {
+    LOCK(self->secondary_select(start, end));
+}
+
+int Fl_Text_Buffer_secondary_selected(Fl_Text_Buffer *self) {
+    LOCK(auto ret = self->secondary_selected());
+    return ret;
+}
+
+void Fl_Text_Buffer_secondary_unselect(Fl_Text_Buffer *self) {
+    LOCK(self->secondary_unselect());
+}
+
+int Fl_Text_Buffer_secondary_selection_position(Fl_Text_Buffer *self, int *start, int *end) {
+    LOCK(auto ret = self->secondary_selection_position(start, end));
+    return ret;
+}
+
+char *Fl_Text_Buffer_secondary_selection_text(Fl_Text_Buffer *self) {
+    LOCK(auto ret = self->secondary_selection_text());
+    return ret;
+}
+
+void Fl_Text_Buffer_remove_secondary_selection(Fl_Text_Buffer *self) {
+    LOCK(self->remove_secondary_selection());
+}
+
+void Fl_Text_Buffer_replace_secondary_selection(Fl_Text_Buffer *self, const char *text) {
+    LOCK(self->replace_secondary_selection(text));
+}
+
 void Fl_Text_Buffer_highlight(Fl_Text_Buffer *self, int start, int end) {
     LOCK(self->highlight(start, end));
 }
@@ -619,6 +671,15 @@ int Fl_Text_Editor_tab_nav(const Fl_Text_Editor *self) {
     return ret;
 }
 
+void Fl_Text_Editor_add_key_binding(Fl_Text_Editor *self, int key, int state,
+                                    int (*kf)(int, Fl_Text_Editor *)) {
+    LOCK(self->add_key_binding(key, state, kf));
+}
+
+void Fl_Text_Editor_remove_key_binding(Fl_Text_Editor *self, int key, int state) {
+    LOCK(self->remove_key_binding(key, state));
+}
+
 WIDGET_CLASS(Fl_Simple_Terminal)
 
 WIDGET_DEFINE(Fl_Simple_Terminal)
